@@ -32,7 +32,9 @@
     #ifdef WINDOWS
         #undef __TEXT
         #include <windows.h>
-        #include <tchar.h>
+        #ifndef WINDOWS_GAMES
+            #include <tchar.h>
+        #endif
     #endif
 #endif //ZENLIB_USEWX
 #ifdef __MINGW32__
@@ -266,7 +268,7 @@ Ztring& Ztring::From_Unicode (const wchar_t* S)
                 if (Size!=0 && Size!=(size_t)-1)
                 {
                     char* AnsiString=new char[Size+1];
-                    Size=wcstombs(AnsiString, S, wcslen(S));
+                    Size=wcstombs(AnsiString, S, Size+1);
                     AnsiString[Size]='\0';
                     assign (AnsiString);
                     delete[] AnsiString;
@@ -649,17 +651,13 @@ Ztring& Ztring::From_ISO_8859_1(const char* S, size_type Start, size_type Length
 
     if (Length==Error)
         Length=strlen(S+Start);
-    #ifdef _UNICODE
-        char* Temp = new char[Length+1];
-        strncpy(Temp, S +Start, Length);
-        Temp[Length] = '\0';
-        From_ISO_8859_1(Temp);
-        delete[] Temp;
-    #else
-        assign(S +Start, Length);
-        if (find(__T('\0')) != std::string::npos)
-            resize(find(__T('\0')));
-    #endif
+
+    char* Temp = new char[Length+1];
+    strncpy(Temp, S +Start, Length);
+    Temp[Length] = '\0';
+    From_ISO_8859_1(Temp);
+    delete[] Temp;
+
     return *this;
 }
 
@@ -688,17 +686,13 @@ Ztring& Ztring::From_ISO_8859_2(const char* S, size_type Start, size_type Length
 
     if (Length==Error)
         Length=strlen(S+Start);
-    #ifdef _UNICODE
-        char* Temp = new char[Length+1];
-        strncpy(Temp, S +Start, Length);
-        Temp[Length] = '\0';
-        From_ISO_8859_2(Temp);
-        delete[] Temp;
-    #else
-        assign(S +Start, Length);
-        if (find(__T('\0')) != std::string::npos)
-            resize(find(__T('\0')));
-    #endif
+
+    char* Temp = new char[Length+1];
+    strncpy(Temp, S +Start, Length);
+    Temp[Length] = '\0';
+    From_ISO_8859_2(Temp);
+    delete[] Temp;
+
     return *this;
 }
 
@@ -744,6 +738,9 @@ Ztring& Ztring::From_CC4 (const int32u S)
         int32u Value=(S&(0xFF<<i))>>i;
         if (Value<0x20)
         {
+            if (!i || (i!=24  && !(S&(0xFFFFFFFF>>(32-i)))))
+                return *this; // Trailing 0 are fine
+
             // Not valid, using 0x as fallback
             clear();
             append(__T("0x"));
@@ -766,6 +763,9 @@ Ztring& Ztring::From_CC3 (const int32u S)
         int32u Value=(S&(0xFF<<i))>>i;
         if (Value<0x20)
         {
+            if (!i || (i!=16  && !(S&(0xFFFFFF>>(24-i)))))
+                return *this; // Trailing 0 are fine
+
             // Not valid, using 0x as fallback
             clear();
             append(__T("0x"));
@@ -1306,7 +1306,28 @@ Ztring& Ztring::Date_From_Seconds_1970 (const int32s Value)
 Ztring& Ztring::Date_From_Seconds_1970 (const int64s Value)
 {
     time_t Time=(time_t)Value;
-    struct tm *Gmt=gmtime(&Time);
+    #if defined(_WIN32)
+        #if _CRT_USE_CONFORMING_ANNEX_K_TIME || (__STDC_WANT_LIB_EXT1__ && defined(__STDC_LIB_EXT1__))
+        // C11 standard version on MSVC or other compilers
+        tm Gmt_Temp;
+        tm* Gmt = gmtime_s(&Time, &Gmt_Temp);
+        #else
+        // MSVC and MinGW-w64 argument order and return value differs from C11 standard
+        tm Gmt_Temp;
+        errno_t gmtime_s_Result = gmtime_s(&Gmt_Temp, &Time);
+        tm* Gmt = gmtime_s_Result ? nullptr : &Gmt_Temp;
+        #endif
+    #elif defined(HAVE_GMTIME_R)
+        // POSIX or C23
+        tm Gmt_Temp;
+        tm *Gmt = gmtime_r(&Time, &Gmt_Temp);
+    #else
+        // Fallback: not thread-safe, but prevents compile errors
+        #ifdef __GNUC__
+        #warning "This version of ZenLib is not thread safe"
+        #endif
+        tm *Gmt = gmtime(&Time);
+    #endif
     if (!Gmt)
     {
         clear();
@@ -1337,9 +1358,32 @@ Ztring& Ztring::Date_From_Seconds_1970 (const int64s Value)
 Ztring& Ztring::Date_From_Seconds_1970_Local (const int32u Value)
 {
     time_t Time=(time_t)Value;
-    struct tm *Gmt=localtime(&Time);
+    #if defined(_WIN32)
+        #if _CRT_USE_CONFORMING_ANNEX_K_TIME || defined(__STDC_LIB_EXT1__)
+        // C11 standard version on MSVC or other compilers
+        tm Gmt_Temp;
+        tm* Gmt = localtime_s(&Time, &Gmt_Temp);
+        #else
+        // MSVC and MinGW-w64 argument order and return value differs from C11 standard
+        tm Gmt_Temp;
+        errno_t localtime_s_Result = localtime_s(&Gmt_Temp, &Time);
+        tm* Gmt = localtime_s_Result ? nullptr : &Gmt_Temp;
+        #endif
+    #elif defined(HAVE_GMTIME_R)
+        // POSIX or C23
+        tm Gmt_Temp;
+        tm *Gmt = localtime_r(&Time, &Gmt_Temp);
+    #else
+        // Fallback: not thread-safe, but prevents compile errors
+        #ifdef __GNUC__
+        #warning "This version of ZenLib is not thread safe"
+        #endif
+        tm *Gmt = localtime(&Time);
+    #endif
     Ztring DateT;
     Ztring Date;
+    if (Gmt)
+    {
     Date+=Ztring::ToZtring((Gmt->tm_year+1900));
     Date+=__T("-");
     DateT.From_Number(Gmt->tm_mon+1); if (DateT.size()<2){DateT=Ztring(__T("0"))+Ztring::ToZtring(Gmt->tm_mon+1);}
@@ -1357,6 +1401,7 @@ Ztring& Ztring::Date_From_Seconds_1970_Local (const int32u Value)
     DateT.From_Number(Gmt->tm_sec); if (DateT.size()<2){DateT=Ztring(__T("0"))+Ztring::ToZtring(Gmt->tm_sec);}
     Date+=DateT;
     assign (Date.c_str());
+    }
     return *this;
 }
 
@@ -1619,18 +1664,23 @@ std::string Ztring::To_UTF8 () const
             case 6:
                 utf8chars[5] = 0x80 | (wc & 0x3f);
                 wc = (wc >> 6) | 0x4000000;
+                [[fallthrough]];
             case 5:
                 utf8chars[4] = 0x80 | (wc & 0x3f);
                 wc = (wc >> 6) | 0x200000;
+                [[fallthrough]];
             case 4:
                 utf8chars[3] = 0x80 | (wc & 0x3f);
                 wc = (wc >> 6) | 0x10000;
+                [[fallthrough]];
             case 3:
                 utf8chars[2] = 0x80 | (wc & 0x3f);
                 wc = (wc >> 6) | 0x800;
+                [[fallthrough]];
             case 2:
                 utf8chars[1] = 0x80 | (wc & 0x3f);
                 wc = (wc >> 6) | 0xc0;
+                [[fallthrough]];
             case 1:
                 utf8chars[0] = (char) wc;
             }
@@ -1827,11 +1877,11 @@ int8s Ztring::To_int8s (int8u Radix, ztring_t Options) const
     #endif
 
     //Rounded
-    if (Options==Ztring_Rounded && find(__T('.'))!=Error)
+    if (Options&Ztring_Rounded && find(__T('.'))!=Error)
     {
         float80 F=To_float80();
         F-=I;
-        if (F>=0.5)
+        if (F>=0.5f)
             return (int8s)I+1;
     }
 
@@ -1865,11 +1915,11 @@ int8u Ztring::To_int8u (int8u Radix, ztring_t Options) const
     #endif
 
     //Rounded
-    if (Options==Ztring_Rounded && find(__T('.'))!=std::string::npos)
+    if (Options&Ztring_Rounded && find(__T('.'))!=std::string::npos)
     {
         float32 F=To_float32();
         F-=I;
-        if (F>=0.5)
+        if (F>=0.5f)
             return (int8u)I+1;
     }
 
@@ -1903,11 +1953,11 @@ int16s Ztring::To_int16s (int8u Radix, ztring_t Options) const
     #endif
 
     //Rounded
-    if (Options==Ztring_Rounded && find(__T('.'))!=Error)
+    if (Options&Ztring_Rounded && find(__T('.'))!=Error)
     {
         float80 F=To_float80();
         F-=I;
-        if (F>=0.5)
+        if (F>=0.5f)
             return (int16s)I+1;
     }
 
@@ -1941,11 +1991,11 @@ int16u Ztring::To_int16u (int8u Radix, ztring_t Options) const
     #endif
 
     //Rounded
-    if (Options==Ztring_Rounded && find(__T('.'))!=std::string::npos)
+    if (Options&Ztring_Rounded && find(__T('.'))!=std::string::npos)
     {
         float32 F=To_float32();
         F-=I;
-        if (F>=0.5)
+        if (F>=0.5f)
             return (int16u)I+1;
     }
 
@@ -1979,11 +2029,11 @@ int32s Ztring::To_int32s (int8u Radix, ztring_t Options) const
     #endif
 
     //Rounded
-    if (Options==Ztring_Rounded && find(__T('.'))!=Error)
+    if (Options&Ztring_Rounded && find(__T('.'))!=Error)
     {
         float80 F=To_float80();
         F-=I;
-        if (F>=0.5)
+        if (F>=0.5f)
             return I+1;
     }
 
@@ -2017,11 +2067,11 @@ int32u Ztring::To_int32u (int8u Radix, ztring_t Options) const
     #endif
 
     //Rounded
-    if (Options==Ztring_Rounded && find(__T('.'))!=std::string::npos)
+    if (Options&Ztring_Rounded && find(__T('.'))!=std::string::npos)
     {
         float32 F=To_float32();
         F-=I;
-        if (F>=0.5)
+        if (F>=0.5f)
             return I+1;
     }
 
@@ -2055,11 +2105,11 @@ int64s Ztring::To_int64s (int8u Radix, ztring_t Options) const
     #endif
 
     //Rounded
-    if (Options==Ztring_Rounded && find(__T('.'))!=std::string::npos)
+    if (Options&Ztring_Rounded && find(__T('.'))!=std::string::npos)
     {
         float32 F=To_float32();
         F-=I;
-        if (F>0.5)
+        if (F>0.5f)
             return I+1;
     }
 
@@ -2093,11 +2143,11 @@ int64u Ztring::To_int64u (int8u Radix, ztring_t Options) const
     #endif
 
     //Rounded
-    if (Options==Ztring_Rounded && find(__T('.'))!=std::string::npos)
+    if (Options&Ztring_Rounded && find(__T('.'))!=std::string::npos)
     {
         float32 F=To_float32();
         F-=I;
-        if (F>=0.5)
+        if (F>=0.5f)
             return I+1;
     }
 
